@@ -35,6 +35,9 @@ import matplotlib.pyplot as plt
 import sys
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigCanvas
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavBar
+from matplotlib.widgets import MultiCursor
+
+import pyperclip
 
 import re
 
@@ -57,6 +60,18 @@ class TagInfoRule():
         
         
 class PlotInfo():
+    '''
+    Contains information about one axes.
+
+    Parameters
+    ----------
+    tagnames : list
+        list of tagnames
+    ax : matplotlib.pyplot.axis
+        the axis this plotinfo is for
+    groupid : string
+        the groupid of this plotinfo/ax
+    '''
     def __init__(self,tagname,groupid,ax):
         self.tagnames = [tagname]
         self.ax = ax
@@ -131,6 +146,8 @@ class PlotManager(QObject):
         self._groupid_plots = {} # dictionary of plotted groupids
         self._taginfo = {} # dictionary of tags
 
+        self.cur = None
+
     def set_dataframe(self,df):
         self.clear_all_plots()
 
@@ -222,23 +239,29 @@ class PlotManager(QObject):
         '''
         Replot the ax in plotinfo.  Used when adding/removing tags
         '''
+        if DEBUG:
+            print("PlotManager::replot()")
 
         if save_xlim:
+            if DEBUG:
+                print("Saving xlim")
             xlim = plotinfo.ax.get_xlim()
 
         plotinfo.ax.clear()
 
-        color = [ self._taginfo[t].color for t in plotinfo.tagnames ]
-        self._df.plot(
-            y=plotinfo.tagnames,
-            color=color,
-            ax=plotinfo.ax,
-            legend=True,
-            include_bool=True,
-        )
+        #color = [ self._taginfo[t].color for t in plotinfo.tagnames ]
+        for tagname in plotinfo.tagnames:
+            plotinfo.ax.plot(
+                self._df[tagname],
+                color=self._taginfo[tagname].color,
+                label=tagname,
+            )
+        plotinfo.ax.legend()
 
         if save_xlim:
             plotinfo.ax.set_xlim(xlim)
+        else:
+            plotinfo.ax.autoscale(axis='x',tight=True)
 
 
 
@@ -262,14 +285,23 @@ class PlotManager(QObject):
             if DEBUG:
                 print("Adding tag to axis")
 
-            xlim = plotinfo.ax.get_xlim()
+            #xlim = plotinfo.ax.get_xlim()
 
+            plotinfo.ax.plot(
+                self._df[tag],
+                color=taginfo.color,
+                label=tag,
+                scalex=False,
+            )
+            plotinfo.ax.legend()
+            '''
             self._df[tag].plot(color=taginfo.color,
                                ax=plotinfo.ax,
                                legend=True,
                                include_bool=True)
+            '''
 
-            plotinfo.ax.set_xlim(xlim)
+            #plotinfo.ax.set_xlim(xlim)
 
         else:
             nplots = len(self._plotinfo)
@@ -295,11 +327,19 @@ class PlotManager(QObject):
                 label=groupid,
                 sharex=sharex
             )
+
+            if DEBUG:
+                print("label_outer for all other axis")
+            for pi in self._plotinfo:
+                pi.ax.tick_params(labelbottom=False)
+
+
             plotinfo = PlotInfo(tag,groupid,ax)
             self._plotinfo.append(plotinfo)
             self._taginfo[tag].plotinfo = plotinfo
             if groupid:
                 self._groupid_plots[groupid] = plotinfo
+
        
             if DEBUG:
                 print("Replotting new axis")
@@ -310,6 +350,12 @@ class PlotManager(QObject):
                 print("Clearing navstack")
             self.plot_window.toolbar._nav_stack.clear()
 
+            self.cur = MultiCursor(
+                self.plot_window.fig.canvas,
+                [ pi.ax for pi in self._plotinfo ],
+                lw=1,
+                color='red')
+
 
     def remove_plot(self,tag):
         taginfo = self._taginfo[tag]
@@ -334,7 +380,9 @@ class PlotManager(QObject):
             if DEBUG:
                 print("Removing axis")
             plotinfo.ax.remove()
+
             self._plotinfo.remove(plotinfo)
+
             if taginfo.groupid in self._groupid_plots:
                 del self._groupid_plots[taginfo.groupid] 
 
@@ -347,7 +395,19 @@ class PlotManager(QObject):
             if len(self._plotinfo) == 0:
                 if DEBUG:
                     print("no more plots left")
-                    self.plot_window.toolbar._nav_stack.clear()
+                self.plot_window.toolbar._nav_stack.clear()
+                self.cur = None
+            
+            else:
+                self.cur = MultiCursor(
+                    self.plot_window.fig.canvas,
+                    [ pi.ax for pi in self._plotinfo ],
+                    lw=1,
+                    color='red')
+
+                self._plotinfo[-1].ax.tick_params(labelbottom=True)
+
+
 
         taginfo.plotinfo = None
 
@@ -376,7 +436,8 @@ class PlotManager(QObject):
 
             self.plot_window.canvas.draw()
         except Exception as e:
-            sys.stderr.write(e)
+            sys.stderr.write('Exception in QtSlot PlotManager::add_remove_plot\n' \
+                + str(e) + '\n')
 
 
     @QtCore.pyqtSlot()
@@ -398,8 +459,8 @@ class PlotManager(QObject):
                     color.append( self._taginfo[tag].color )
 
                 xlim = plotinfo.ax.get_xlim()
-                x0 = pandas.Timestamp(xlim[0],unit='m')
-                x1 = pandas.Timestamp(xlim[1],unit='m')
+                x0 = matplotlib.dates.num2date(xlim[0])
+                x1 = matplotlib.dates.num2date(xlim[1])
                 ylim = plotinfo.ax.get_ylim()
 
                 code += 'df.plot(\n' + \
@@ -421,7 +482,11 @@ class PlotManager(QObject):
 
             code += 'fig.tight_layout()\n'
 
-        print(code)
+        #print(code)
+        pyperclip.copy(code)
+
+        code = ("<b>The following is copied to your clipboard:</b><br/>"
+                + code.replace('\n','<br/>').replace(" ",'&nbsp;') )
 
         QMessageBox.information(None, "Show Me",
                                       code,
@@ -569,6 +634,15 @@ class ToolPanel(QWidget):
         if DEBUG:
             print("ToolPanel::clear_clicked")
 
+        ans = QMessageBox.question(
+            None,
+            "Confirm clear","Are you sure you want to clear all plots?"
+        )
+        if (ans != QMessageBox.Yes):
+            if DEBUG:
+                print("Didn't click yes on the messagebox")
+            return
+
         for tool in self._tools:
             try:
                 tool.reset()
